@@ -33,7 +33,7 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGES.
  *
- * $Id: if_athvar.h 1657 2006-06-28 06:45:43Z proski $
+ * $Id: if_athvar.h 2104 2007-02-13 05:54:52Z proski $
  */
 
 /*
@@ -43,6 +43,7 @@
 #define _DEV_ATH_ATHVAR_H
 
 #include "ah.h"
+#include "ah_os.h"
 #include "if_athioctl.h"
 #include "net80211/ieee80211.h"		/* XXX for WME_NUM_AC */
 
@@ -71,22 +72,23 @@ typedef void *TQUEUE_ARG;
 #include <linux/sched.h>
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,5,41)
 #include <linux/tqueue.h>
-#define ATH_WORK_THREAD			tq_struct
-#define ATH_SCHEDULE_TASK(t)		schedule_task((t))
-#define ATH_INIT_SCHED_TASK(t, f, d) do { 	\
-	memset((t),0,sizeof(struct tq_struct)); 	\
+#define work_struct			tq_struct
+#define schedule_work(t)		schedule_task((t))
+#define flush_scheduled_work()		flush_scheduled_tasks()
+#define ATH_INIT_WORK(t, f) do { 			\
+	memset((t),0,sizeof(struct tq_struct)); \
 	(t)->routine = (void (*)(void*)) (f); 	\
-	(t)->data=(void *) (d); 		\
+	(t)->data=(void *) (t);			\
 } while (0)
-#define ATH_FLUSH_TASKS			flush_scheduled_tasks
 #else
 #include <linux/workqueue.h>
-#define ATH_SCHEDULE_TASK(t)		schedule_work((t))
 
-#define ATH_INIT_SCHED_TASK(_t, _f, _d)	INIT_WORK((_t), (void (*)(void *))(_f), (void *)(_d));
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,20)
+#define ATH_INIT_WORK(_t, _f)	INIT_WORK((_t), (void (*)(void *))(_f), (_t));
+#else
+#define ATH_INIT_WORK(_t, _f)	INIT_WORK((_t), (_f));
+#endif
 
-#define ATH_WORK_THREAD			work_struct
-#define	ATH_FLUSH_TASKS			flush_scheduled_work
 #endif /* KERNEL_VERSION < 2.5.41 */
 
 /*
@@ -108,6 +110,50 @@ typedef void irqreturn_t;
 #define	SET_NETDEV_DEV(ndev, pdev)
 #endif
 
+#ifdef to_net_dev
+#define ATH_GET_NETDEV_DEV(ndev)	((ndev)->dev.parent)
+#else
+#define ATH_GET_NETDEV_DEV(ndev)	((ndev)->class_dev.dev)
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,23)
+static inline struct net_device *_alloc_netdev(int sizeof_priv, const char *mask,
+					       void (*setup)(struct net_device *))
+{
+	struct net_device *dev;
+	int alloc_size;
+
+	/* ensure 32-byte alignment of the private area */
+	alloc_size = sizeof (*dev) + sizeof_priv + 31;
+
+	dev = (struct net_device *) kmalloc (alloc_size, GFP_KERNEL);
+	if (dev == NULL)
+	{
+		printk(KERN_ERR "alloc_dev: Unable to allocate device memory.\n");
+		return NULL;
+	}
+
+	memset(dev, 0, alloc_size);
+
+	if (sizeof_priv)
+		dev->priv = (void *) (((long)(dev + 1) + 31) & ~31);
+
+	setup(dev);
+	strcpy(dev->name, mask);
+
+	return dev;
+}
+
+/* Avoid name collision - some vendor kernels backport alloc_netdev() */
+#undef alloc_netdev
+#define alloc_netdev(s,m,d) _alloc_netdev(s,m,d)
+
+static inline struct proc_dir_entry *PDE(const struct inode *inode)
+{
+	return (struct proc_dir_entry *)inode->u.generic_ip;
+}
+#endif
+
 
 /*
  * Macro to expand scalars to 64-bit objects
@@ -240,7 +286,7 @@ typedef void irqreturn_t;
 #define A_MAX(a,b) ((a) > (b) ? (a) : (b))
 
 /*
- * Macros to obtain the Group Poll Periodicty in various situations
+ * Macros to obtain the Group Poll Periodicity in various situations
  *
  * Curerntly there are the two cases
  * (a) When there are no XR STAs associated
@@ -267,8 +313,8 @@ typedef void irqreturn_t;
  */
 #define	ATH_KEYMAX	128		/* max key cache size we handle */
 #define	ATH_KEYBYTES	(ATH_KEYMAX / NBBY)	/* storage space in bytes */
-#define	ATH_MIN_FF_RATE	12000		/* min rate fof ff aggragattion.in Kbps  */
-#define	ATH_MIN_FF_RATE	12000		/* min rate fof ff aggragattion.in Kbps  */
+#define	ATH_MIN_FF_RATE	12000		/* min rate for ff aggregation in kbps */
+#define	ATH_MIN_FF_RATE	12000		/* min rate for ff aggregation in kbps */
 struct ath_buf;
 typedef STAILQ_HEAD(, ath_buf) ath_bufhead;
 
@@ -278,14 +324,14 @@ struct ath_node {
 	u_int16_t an_decomp_index; 		/* decompression mask index */
 	u_int32_t an_avgrssi;			/* average rssi over all rx frames */
 	u_int8_t  an_prevdatarix;		/* rate ix of last data frame */
-	u_int16_t an_minffrate;			/* mimum rate in kbps for ff to aggragate */
+	u_int16_t an_minffrate;			/* min rate in kbps for ff to aggregate */
 	HAL_NODE_STATS an_halstats;		/* rssi statistics used by hal */
 	struct ath_buf *an_tx_ffbuf[WME_NUM_AC]; /* ff staging area */
 	ath_bufhead an_uapsd_q;			/* U-APSD delivery queue */
 	int an_uapsd_qdepth; 			/* U-APSD delivery queue depth */
 	ath_bufhead an_uapsd_overflowq; 	/* U-APSD overflow queue (for > MaxSp frames) */
 	int an_uapsd_overflowqdepth; 		/* U-APSD overflow queue depth */
-	spinlock_t an_uapsd_lock; 		/* U-APSD deleivery queue lock */
+	spinlock_t an_uapsd_lock; 		/* U-APSD delivery queue lock */
 	/* variable-length rate control state follows */
 };
 #define	ATH_NODE(_n)			((struct ath_node *)(_n))
@@ -331,13 +377,13 @@ struct ath_buf {
 	dma_addr_t bf_skbaddr;			/* physical addr of skb data */
 	struct ieee80211_node *bf_node;		/* pointer to the node */
 	u_int32_t bf_status;			/* status flags */
+	u_int16_t bf_flags;			/* tx descriptor flags */
 #ifdef ATH_SUPERG_FF
-	/* XXX: combine this with bf_skbaddr if it ever changes to accomodate
+	/* XXX: combine this with bf_skbaddr if it ever changes to accommodate
 	 *      multiple segments.
 	 */
-	u_int32_t bf_queueage; 			/* "age" of txq when this buffer placed on stageq */
 	u_int16_t bf_numdesc;			/* number of descs used */
-	u_int16_t bf_flags;			/* tx descriptor flags */
+	u_int32_t bf_queueage; 			/* "age" of txq when this buffer placed on stageq */
 	dma_addr_t bf_skbaddrff[ATH_TXDESC-1]; 	/* extra addrs for ff */
 #endif
 };
@@ -467,11 +513,12 @@ struct ath_vap {
  */
 #define ATH_TXQ_MOVE_Q(_tqs,_tqd)  ATH_TXQ_MOVE_MCASTQ(_tqs,_tqd)
 
-#define	BSTUCK_THRESH	3	/* # of stuck beacons before resetting NB: this is a guess*/
+#define	BSTUCK_THRESH	10	/* # of stuck beacons before resetting NB: this is a guess*/
 
 struct ath_softc {
 	struct ieee80211com sc_ic;		/* NB: must be first */
 	struct net_device *sc_dev;
+	void __iomem *sc_iobase;		/* address of the device */
 	struct semaphore sc_lock;		/* dev-level lock */
 	struct net_device_stats	sc_devstats;	/* device statistics */
 	struct ath_stats	sc_stats;		/* private statistics */
@@ -499,7 +546,7 @@ struct ath_softc {
 			sc_dturbo_switch:1,	/* turbo switch mode*/
 			sc_dturbo_hold:1,	/* dynamic turbo hold state */
 			sc_rate_recn_state:1,	/* dynamic turbo state recmded by ratectrl */
-			sc_ignore_ar:1,		/* ignore AR during transision*/
+			sc_ignore_ar:1,		/* ignore AR during transition */
 			sc_ledstate:1,		/* LED on/off state */
 			sc_blinking:1,		/* LED blink operation active */
 			sc_beacons:1,		/* beacons running */
@@ -613,12 +660,10 @@ struct ath_softc {
 
 	struct timer_list sc_cal_ch;		/* calibration timer */
 	HAL_NODE_STATS sc_halstats;		/* station-mode rssi stats */
-	struct ATH_WORK_THREAD sc_radartask;	/* Schedule task for DFS handling */
+	struct work_struct sc_radartask;	/* Schedule task for DFS handling */
 
-#ifdef CONFIG_SYSCTL
 	struct ctl_table_header *sc_sysctl_header;
 	struct ctl_table *sc_sysctls;
-#endif
 
 	u_int16_t sc_reapcount;  		/* # of tx buffers reaped after net dev stopped */
 
@@ -635,6 +680,7 @@ struct ath_softc {
 	u_int32_t sc_dturbo_bw_turbo;		/* bandwidth threshold */
 #endif
 	u_int sc_slottimeconf;			/* manual override for slottime */
+	int16_t sc_channoise; 			/* Measured noise of current channel (dBm) */
 };
 
 typedef void (*ath_callback) (struct ath_softc *);
@@ -681,18 +727,19 @@ typedef void (*ath_callback) (struct ath_softc *);
 #define	ATH_LOCK(_sc)			down(&(_sc)->sc_lock)
 #define	ATH_UNLOCK(_sc)			up(&(_sc)->sc_lock)
 
-int ath_attach(u_int16_t, struct net_device *);
+int ath_attach(u_int16_t, struct net_device *, HAL_BUS_TAG);
 int ath_detach(struct net_device *);
 void ath_resume(struct net_device *);
 void ath_suspend(struct net_device *);
-void ath_shutdown(struct net_device *);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,19)
+irqreturn_t ath_intr(int, void *);
+#else
 irqreturn_t ath_intr(int, void *, struct pt_regs *);
+#endif
 int ath_ioctl_ethtool(struct ath_softc *, int, void __user *);
 void bus_read_cachesize(struct ath_softc *, u_int8_t *);
-#ifdef CONFIG_SYSCTL
 void ath_sysctl_register(void);
 void ath_sysctl_unregister(void);
-#endif /* CONFIG_SYSCTL */
 
 /*
  * HAL definitions to comply with local coding convention.
@@ -850,8 +897,12 @@ void ath_sysctl_unregister(void);
 	ath_hal_getcapability(_ah, HAL_CAP_REG_DMN, 0, (_prd))
 #define	ath_hal_getcountrycode(_ah, _pcc) \
 	(*(_pcc) = (_ah)->ah_countryCode)
-#define	ath_hal_tkipsplit(_ah) \
+#define ath_hal_hastkipsplit(_ah) \
 	(ath_hal_getcapability(_ah, HAL_CAP_TKIP_SPLIT, 0, NULL) == HAL_OK)
+#define ath_hal_gettkipsplit(_ah) \
+	(ath_hal_getcapability(_ah, HAL_CAP_TKIP_SPLIT, 1, NULL) == HAL_OK)
+#define ath_hal_settkipsplit(_ah, _v) \
+	ath_hal_setcapability(_ah, HAL_CAP_TKIP_SPLIT, 1, _v, NULL)
 #define	ath_hal_wmetkipmic(_ah) \
 	(ath_hal_getcapability(_ah, HAL_CAP_WME_TKIPMIC, 0, NULL) == HAL_OK)
 #define	ath_hal_hwphycounters(_ah) \
@@ -962,5 +1013,7 @@ void ath_sysctl_unregister(void);
 	((*(_ah)->ah_dfsNolCheck)((_ah), (_chan), (_nchans)))
 #define ath_hal_radar_wait(_ah, _chan) \
 	((*(_ah)->ah_radarWait)((_ah), (_chan)))
+#define ath_hal_get_channel_noise(_ah, _chan) \
+	((*(_ah)->ah_getChanNoise)((_ah), (_chan)))
 
 #endif /* _DEV_ATH_ATHVAR_H */
